@@ -69,22 +69,25 @@ class Dialog(MDialog, Window):
         buttons = QtGui.QDialogButtonBox()
 
         # 'OK' button.
-        # FIXME v3: Review how this is supposed to work for non-modal dialogs
-        # (ie. how does anything act on a button click?)
         if self.ok_label:
-            btn = buttons.addButton(self.ok_label, QtGui.QDialogButtonBox.AcceptRole)
+            btn = buttons.addButton(self.ok_label, 
+                                    QtGui.QDialogButtonBox.AcceptRole)
         else:
             btn = buttons.addButton(QtGui.QDialogButtonBox.Ok)
 
         btn.setDefault(True)
+        QtCore.QObject.connect(btn, QtCore.SIGNAL('clicked()'), 
+                               self.control, QtCore.SLOT('accept()'))
 
         # 'Cancel' button.
-        # FIXME v3: Review how this is supposed to work for non-modal dialogs
-        # (ie. how does anything act on a button click?)
         if self.cancel_label:
-            buttons.addButton(self.cancel_label, QtGui.QDialogButtonBox.RejectRole)
+            btn = buttons.addButton(self.cancel_label, 
+                                    QtGui.QDialogButtonBox.RejectRole)
         else:
-            buttons.addButton(QtGui.QDialogButtonBox.Cancel)
+            btn = buttons.addButton(QtGui.QDialogButtonBox.Cancel)
+
+        QtCore.QObject.connect(btn, QtCore.SIGNAL('clicked()'), 
+                               self.control, QtCore.SLOT('reject()'))
 
         # 'Help' button.
         # FIXME v3: In the original code the only possible hook into the help
@@ -100,15 +103,19 @@ class Dialog(MDialog, Window):
         return buttons
 
     def _create_contents(self, parent):
-        lay = QtGui.QVBoxLayout()
+        layout = QtGui.QVBoxLayout()
 
-        lay.addWidget(self._create_dialog_area(parent))
-        lay.addWidget(self._create_buttons(parent))
+        if not self.resizeable:
+            layout.setSizeConstraint(QtGui.QLayout.SetFixedSize)
 
-        parent.setLayout(lay)
+        layout.addWidget(self._create_dialog_area(parent))
+        layout.addWidget(self._create_buttons(parent))
+
+        parent.setLayout(layout)
 
     def _create_dialog_area(self, parent):
         panel = QtGui.QWidget(parent)
+        panel.setMinimumSize(QtCore.QSize(100, 200))
 
         palette = panel.palette()
         palette.setColor(QtGui.QPalette.Window, QtGui.QColor('red'))
@@ -128,13 +135,27 @@ class Dialog(MDialog, Window):
     def _create_control(self, parent):
         dlg = QtGui.QDialog(parent)
 
+        # Setting return code and firing close events is handled for 'modal' in 
+        # MDialog's open method. For 'nonmodal', we do it here.
+        if self.style == 'nonmodal':
+            QtCore.QObject.connect(dlg, QtCore.SIGNAL('finished(int)'),
+                                   self._finished_fired)
+
         if self.size != (-1, -1):
             dlg.resize(*self.size)
 
-        # FIXME v3: Decide what to do with the resizable trait (ie. set the
-        # size policy).
         dlg.setWindowTitle(self.title)
 
         return dlg
 
+    ###########################################################################
+    # Private interface.
+    ###########################################################################
+
+    def _finished_fired(self, result):
+        """ Called when the dialog is closed (and nonmodal). """
+
+        self.return_code = _RESULT_MAP[result]
+        self.close()
+
 #### EOF ######################################################################
