@@ -21,6 +21,7 @@ from enthought.traits.api import Tuple
 # Local imports.
 from enthought.pyface.i_window import IWindow, MWindow
 from enthought.pyface.key_pressed_event import KeyPressedEvent
+from gui import GUI
 from widget import Widget
 
 
@@ -156,7 +157,8 @@ class _EventFilter(QtCore.QObject):
             return False
 
         if e.type() == QtCore.QEvent.Close:
-            window.close()
+            # Do not destroy the window during its event handler.
+            GUI.invoke_later(window.close)
 
             if window.control is not None:
                 e.ignore()
@@ -168,16 +170,19 @@ class _EventFilter(QtCore.QObject):
                 window.activated = window
             else:
                 window.deactivated = window
+
         elif e.type() == QtCore.QEvent.Resize:
             # Get the new size and set the shadow trait without performing
             # notification.
             size = e.size()
             window._size = (size.width(), size.height())
+
         elif e.type() == QtCore.QEvent.Move:
             # Get the real position and set the trait without performing
             # notification.
             pos = e.pos()
             window._position = (pos.x(), pos.y())
+
         elif e.type() == QtCore.QEvent.KeyPress:
             # Pyface doesn't seem to be Unicode aware.  Only keep the key code
             # if it corresponds to a single Latin1 character.
