@@ -45,6 +45,20 @@ class Wizard(MWizard, Dialog):
 
     title = Unicode('Wizard')
 
+    ###########################################################################
+    # 'IWizard' interface.
+    ###########################################################################
+
+    # Override MWizard implementation to do nothing. We still call these methods
+    # because it expected by IWizard, and users may wish to hook in custom code
+    # before changing a page.
+
+    def next(self):
+        pass
+
+    def previous(self):
+        pass
+
     ###########################################################################
     # Protected 'IDialog' interface.
     ###########################################################################
@@ -57,14 +71,20 @@ class Wizard(MWizard, Dialog):
     ###########################################################################
 
     def _create_control(self, parent):
-        control = _Wizard(parent, self.controller)
-
-        control.setWindowTitle(self.title)
+        control = _Wizard(parent, self)
+        control.setOptions(QtGui.QWizard.NoDefaultButton |
+                           QtGui.QWizard.NoBackButtonOnStartPage)
         control.setModal(self.style == 'modal')
+        control.setWindowTitle(self.title)
+
+        # Necessary for 'nonmodal'. See Dialog for more info.
+        if self.style == 'nonmodal':
+            QtCore.QObject.connect(control, QtCore.SIGNAL('finished(int)'),
+                                   self._finished_fired)
 
         if self.size != (-1, -1):
             size = QtCore.QSize(*self.size)
-            control.setMaximumSize(size)
+            control.setMinimumSize(size)
             control.resize(size)
 
         if not self.show_cancel:
@@ -77,6 +97,7 @@ class Wizard(MWizard, Dialog):
 
         # Add the initial pages.
         for page in self.pages:
+            page.pyface_wizard = self
             control.addWizardPage(page)
 
         # Set the start page.
@@ -139,12 +160,13 @@ class _Wizard(QtGui.QWizard):
     """ A QWizard sub-class that hooks into the controller to determine the
     next page to show. """
 
-    def __init__(self, parent, controller):
+    def __init__(self, parent, pyface_wizard):
         """ Initialise the object. """
 
         QtGui.QWizard.__init__(self, parent)
 
-        self._controller = controller
+        self._pyface_wizard = pyface_wizard
+        self._controller = pyface_wizard.controller
         self._ids = {}
 
         QtCore.QObject.connect(self, QtCore.SIGNAL('currentIdChanged(int)'),
@@ -155,6 +177,7 @@ class _Wizard(QtGui.QWizard):
 
         # We must pass a parent otherwise TraitsUI does the wrong thing.
         qpage = page.create_page(self)
+        qpage.pyface_wizard = self._pyface_wizard
         id = self.addPage(qpage)
         self._ids[id] = page
 
