@@ -47,7 +47,8 @@ class Editor(MEditor):
         """ Destroy the toolkit-specific control that represents the part. """
 
         if self.control is not None:
-            self.control.setParent(None)
+            self.control.hide()
+            self.control.deleteLater()
             self.control = None
 
         return
