@@ -7,10 +7,12 @@
 
 #------------------------------------------------------------------------------
 
+# Standard library imports.
+import sys
+
 # Major library imports.
 from PyQt4 import QtCore, QtGui
 
-
 class SplitTabWidget(QtGui.QSplitter):
     """ The SplitTabWidget class is a hierarchy of QSplitters the leaves of
     which are QTabWidgets.  Any tab may be moved around with the hierarchy
@@ -36,12 +38,19 @@ class SplitTabWidget(QtGui.QSplitter):
         QtCore.QObject.connect(QtGui.qApp,
                 QtCore.SIGNAL('focusChanged(QWidget *,QWidget *)'),
                 self._focus_changed)
+        
+        # default close tab callback just closes a control
+        self.close_tab_request_callback = lambda w: w.close()
 
     def clear(self):
         """ Restore the widget to its pristine state. """
 
+        w = None
         for i in range(self.count()):
-            self.widget(i).setParent(None)
+            w = self.widget(i)
+            w.hide()
+            w.deleteLater()
+        del w
 
         self._repeat_focus_changes = True
         self._rband = None
@@ -167,6 +176,11 @@ class SplitTabWidget(QtGui.QSplitter):
         if ch is not self._current_tab_w:
             self._set_current_tab(ch, idx)
             ch.tabBar().setFocus()
+    
+    def _close_tab_request(self, w):
+        """ A close button was clicked in one of out _TabWidgets """
+        
+        self.close_tab_request_callback(w)
 
     def setCurrentWidget(self, w):
         """ Make the given widget current. """
@@ -184,8 +198,19 @@ class SplitTabWidget(QtGui.QSplitter):
         if tw is not None:
             if icon is None:
                 icon = tw.active_icon()
-
+            
             tw.setTabIcon(tidx, icon)
+    
+    def setTabTextColor(self, w, color=None):
+        """ Set the tab text color on a particular widget w
+        """
+        tw, tidx = self._tab_widget(w)
+
+        if tw is not None:
+            if color is None:
+                # null color reverts to foreground role color
+                color = QtGui.QColor()
+            tw.tabBar().setTabTextColor(tidx, color)
 
     def setWidgetTitle(self, w, title):
         """ Set the title for the given widget. """
@@ -272,15 +297,16 @@ class SplitTabWidget(QtGui.QSplitter):
         # See if the widget that has lost the focus is ours.
         otw, _ = self._tab_widget_of(old)
 
-        if otw is ntw:
-            return
+        # XXX this prevents focus changes between different tabs of the same tabwidget
+        #if otw is ntw:
+        #    return
 
         if otw is not None or ntw is not None:
             if ntw is None:
                 nw = None
             else:
                 nw = ntw.widget(ntidx)
-
+    
             self.emit(QtCore.SIGNAL('hasFocus'), nw)
 
     def _tab_widget_of(self, target):
@@ -385,10 +411,11 @@ class SplitTabWidget(QtGui.QSplitter):
 
             QtGui.qApp.blockSignals(True)
 
-            ticon, ttext, twidg = self._remove_tab(stab_w, stab)
+            ticon, ttext, ttextcolor, twidg = self._remove_tab(stab_w, stab)
 
             if dhs == self._HS_AFTER_LAST_TAB:
                 idx = dtab_w.addTab(twidg, ticon, ttext)
+                dtab_w.tabBar().setTabTextColor(idx, ttextcolor)
             elif dtab_w is stab_w:
                 # Adjust the index if necessary in case the removal of the tab
                 # from its old position has skewed things.
@@ -398,8 +425,10 @@ class SplitTabWidget(QtGui.QSplitter):
                     dst -= 1
 
                 idx = dtab_w.insertTab(dst, twidg, ticon, ttext)
+                dtab_w.tabBar().setTabTextColor(idx, ttextcolor)
             else:
                 idx = dtab_w.insertTab(dhs, twidg, ticon, ttext)
+                dtab_w.tabBar().setTabTextColor(idx, ttextcolor)
 
             # Make the tab current in its new position.
             self._set_current_tab(dtab_w, idx)
@@ -412,9 +441,10 @@ class SplitTabWidget(QtGui.QSplitter):
 
             # Remove the tab from its current tab widget and create a new one
             # for it.
-            ticon, ttext, twidg = self._remove_tab(stab_w, stab)
+            ticon, ttext, ttextcolor, twidg = self._remove_tab(stab_w, stab)
             new_tw = _TabWidget(self)
             new_tw.addTab(twidg, ticon, ttext)
+            new_tw.tabBar().setTabTextColor(0, ttextcolor)
 
             # Get the splitter containing the destination tab widget.
             dspl = dtab_w.parent()
@@ -498,10 +528,11 @@ class SplitTabWidget(QtGui.QSplitter):
 
         icon = tab_w.tabIcon(tab)
         text = tab_w.tabText(tab)
+        text_color = tab_w.tabBar().tabTextColor(tab)
         w = tab_w.widget(tab)
         tab_w.removeTab(tab)
 
-        return (icon, text, w)
+        return (icon, text, text_color, w)
 
     def _hotspot(self, pos):
         """ Return a tuple of the tab widget, hotspot and hostspot geometry (as
@@ -597,9 +628,39 @@ class SplitTabWidget(QtGui.QSplitter):
                     return (tw, self._HS_AFTER_LAST_TAB, (gx, gy, w, h))
 
                 return (tw, i + 1, (gx, gy, w, h))
+        else:
+            rect = tab_bar.rect()
+            if rect.contains(tpos):
+                gpos = tab_bar.mapToGlobal(rect.topLeft())
+                gx = gpos.x()
+                gy = gpos.y()
+                w = rect.width() 
+                h = rect.height()
+                if top_bottom:
+                    tab_widths = sum(tab_bar.tabRect(i).width()
+                        for i in range(tab_bar.count()))
+                    w -= tab_widths
+                    gx += tab_widths
+                else:
+                    tab_heights = sum(tab_bar.tabRect(i).height()
+                        for i in range(tab_bar.count()))
+                    h -= tab_heights
+                    gy -= tab_heights
+                return (tw, self._HS_AFTER_LAST_TAB, (gx, gy, w, h))
+                
+                
 
         return miss
 
+active_style = """QTabWidget::pane { /* The tab widget frame */
+     border: 2px solid #00FF00;
+ }
+"""
+inactive_style = """QTabWidget::pane { /* The tab widget frame */
+     border: 2px solid #C2C7CB;
+     margin: 0px;
+ }
+"""
 
 class _TabWidget(QtGui.QTabWidget):
     """ The _TabWidget class is a QTabWidget with a dragable tab bar. """
@@ -611,6 +672,11 @@ class _TabWidget(QtGui.QTabWidget):
         """ Initialise the instance. """
 
         QtGui.QTabWidget.__init__(self, *args)
+        
+        # XXX this requires Qt > 4.5
+        if sys.platform == 'darwin':
+            self.setDocumentMode(True)
+        #self.setStyleSheet(inactive_style)
 
         self._root = root
 
@@ -618,10 +684,8 @@ class _TabWidget(QtGui.QTabWidget):
         # in PyQt v4.2 and earlier.
         self.setTabBar(_DragableTabBar(self._root, self))
 
-        # Add the button used to close the current tab.
-        buttn = _TabCloseButton(self)
-        self.connect(buttn, QtCore.SIGNAL('clicked()'), self._close_tab)
-        self.setCornerWidget(buttn)
+        self.setTabsClosable(True)        
+        self.tabCloseRequested.connect(self._close_tab)        
 
     def active_icon(self):
         """ Return the QIcon to be used to indicate an active tab page. """
@@ -641,7 +705,7 @@ class _TabWidget(QtGui.QTabWidget):
             p.begin(pm)
 
             # Fill the image background from the tab background.
-            p.initFrom(self)
+            p.initFrom(self.tabBar())
             p.fillRect(0, 0, width, height, p.background())
 
             # Create the colour gradient.
@@ -688,11 +752,12 @@ class _TabWidget(QtGui.QTabWidget):
         if self._root._current_tab_w is self and self._root._current_tab_idx == idx:
             self._root._current_tab_w = None
 
-    def _close_tab(self):
+    def _close_tab(self, index):
         """ Close the current tab. """
+        # XXX want to call editor.close() to give editor a chance to react...
 
-        self.currentWidget().close()
-
+        self._root._close_tab_request(self.widget(index))
+        
 
 class _TabCloseButton(QtGui.QAbstractButton):
     """ The _TabCloseButton class implements a button that is intended to close
@@ -806,6 +871,10 @@ class _DragableTabBar(QtGui.QTabBar):
 
         QtGui.QTabBar.__init__(self, parent)
 
+        # XXX this requires Qt > 4.5
+        if sys.platform == 'darwin':
+            self.setDocumentMode(True)
+
         self._root = root
         self._drag_state = None
 
@@ -918,6 +987,10 @@ class _DragState(object):
         tab = self._tab
 
         ctb = self._clone = QtGui.QTabBar()
+        # XXX this requires Qt > 4.5
+        if sys.platform == 'darwin':
+            self.setDocumentMode(True)
+            
         ctb.setWindowFlags(QtCore.Qt.FramelessWindowHint |
                            QtCore.Qt.Tool |
                            QtCore.Qt.X11BypassWindowManagerHint)
