@@ -21,7 +21,6 @@ from PyQt4 import QtCore, QtGui
 from enthought.traits.api import Instance, on_trait_change
 
 # Local imports.
-from editor import Editor
 from split_tab_widget import SplitTabWidget
 from enthought.pyface.message_dialog import error
 from enthought.pyface.workbench.i_workbench_window_layout import \
@@ -128,6 +127,7 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
 
     def close(self):
         self._qt4_editor_area.clear()
+        self._qt4_editor_area.close_tab_request_callback = None
 
         # Delete all dock widgets.
         for v in self.window.views:
@@ -146,6 +146,8 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
         QtCore.QObject.connect(self._qt4_editor_area,
                 QtCore.SIGNAL('focusChanged(QWidget *,QWidget *)'),
                 self._qt4_view_focus_changed)
+        
+        self._qt4_editor_area.close_tab_request_callback = self._qt4_tab_close_request
 
         return self._qt4_editor_area
 
@@ -285,21 +287,23 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
         """ Handle an editor getting the focus. """
 
         for editor in self.window.editors:
-            editor.has_focus = (editor.control is new)
+            editor.has_focus = (editor.control is new) or (new in editor.control.children())
 
     @on_trait_change('window.active_editor')
     def _qt4_active_editor_changed(self, obj, trait_name, old, new):
         """ Handle the change of active editor. """
-
+        
         # Do we need to do this verification?
         if obj is not self.window or trait_name != 'active_editor':
             return
 
         for editor in self.window.editors:
             if editor is old:
-                self._qt4_editor_area.setActiveIcon(editor.control, QtGui.QIcon())
+                tw, tidx = self._qt4_editor_area._tab_widget(editor.control)
+                tw.setTabText(tidx, editor.name)
             elif editor is new:
-                self._qt4_editor_area.setActiveIcon(editor.control)
+                tw, tidx = self._qt4_editor_area._tab_widget(editor.control)
+                tw.setTabText(tidx, u'\u25b6' + editor.name)
 
     def _qt4_view_focus_changed(self, old, new):
         """ Handle the change of focus for a view. """
@@ -320,6 +324,14 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
                 if view is not focus_part and view.control is not None and view.control.isAncestorOf(old):
                     view.has_focus = False
                     break
+    
+    def _qt4_tab_close_request(self, control):
+        """ Handle a tabCloseRequest from the splitter widget. """
+        
+        for editor in self.window.editors:
+            if editor.control == control:
+                editor.close()
+                break
 
     def _qt4_get_editor_control(self, editor):
         """ Create the editor control if it hasn't already been done. """
@@ -428,7 +440,8 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
             except:
                 # Tidy up if the view couldn't be created.
                 delattr(view, '_qt4_dock')
-                dw.setParent(None)
+                self.window.control.removeDockWidget(dw)
+                dw.deleteLater()
                 del dw
                 raise
 
@@ -449,7 +462,8 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
         delattr(view, '_qt4_dock')
 
         # Delete the dock (and the view container).
-        dw.setParent(None)
+        self.window.control.removeDockWidget(dw)
+        dw.deleteLater()
 
     def _qt4_handle_dock_visibility(self, checked):
         """ Handle the visibility of a dock window changing. """
@@ -470,16 +484,19 @@ class WorkbenchWindowLayout(MWorkbenchWindowLayout):
         with standard margins.
         """
         lay = w.layout()
+        
+        # XXX I don't think we should do this... CJW
 
-        if lay is not None:
-            lay.setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
+        #if lay is not None:
+        #    lay.setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
 
-            sty = w.style()
-            l = sty.pixelMetric(QtGui.QStyle.PM_LayoutLeftMargin)
-            t = sty.pixelMetric(QtGui.QStyle.PM_LayoutTopMargin)
-            r = sty.pixelMetric(QtGui.QStyle.PM_LayoutRightMargin)
-            b = sty.pixelMetric(QtGui.QStyle.PM_LayoutBottomMargin)
-            lay.setContentsMargins(l, t, r, b)
+        #    sty = w.style()
+        #    l = sty.pixelMetric(QtGui.QStyle.PM_LayoutLeftMargin)
+        #    t = sty.pixelMetric(QtGui.QStyle.PM_LayoutTopMargin)
+        #    r = sty.pixelMetric(QtGui.QStyle.PM_LayoutRightMargin)
+        #    b = sty.pixelMetric(QtGui.QStyle.PM_LayoutBottomMargin)
+        #    lay.setContentsMargins(l, t, r, b)
+        #    lay.setContentsMargins(0, 0, 0, 0)
 
     def _qt4_monitor(self, control):
         """ Install an event filter for a view or editor control to keep an eye
