@@ -23,3 +23,11 @@ import toolkit
 
 # Reference to the GUIToolkit object for PyQt.
 toolkit = toolkit.GUIToolkit()
+
+# For py2app / py2exe support
+try:
+    import modulefinder
+    for p in __path__:
+        modulefinder.AddPackagePath(__name__, p)
+except:
+    pass
