@@ -21,6 +21,9 @@ from string import capitalize
     
 from PyQt4 import QtCore, QtGui
 
+from enthought.traits.api \
+    import Bool, Property
+
 # FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
 # compatibility. The class has been moved to the 
 # enthought.traits.ui.editors.enum_editor file.
@@ -35,9 +38,6 @@ from constants \
     
 from helper \
     import enum_values_changed
-    
-from enthought.traits.api \
-    import Property
            
 #-------------------------------------------------------------------------------
 #  'BaseEditor' class:
@@ -182,22 +182,35 @@ class SimpleEditor ( BaseEditor ):
         """
         super( SimpleEditor, self ).init( parent )
         
-        self.control = control = QtGui.QComboBox()
-        control.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        self.control = control = self.create_combo_box()
         control.addItems(self.names)
+
         QtCore.QObject.connect(control,
-                QtCore.SIGNAL('currentIndexChanged(QString)'),
-                self.update_object)
+                               QtCore.SIGNAL('currentIndexChanged(QString)'),
+                               self.update_object)
 
         if self.factory.evaluate is not None:
             control.setEditable(True)
             QtCore.QObject.connect(control,
-                    QtCore.SIGNAL('editTextChanged(QString)'),
-                    self.update_text_object)
+                                   QtCore.SIGNAL('editTextChanged(QString)'),
+                                   self.update_text_object)
 
         self._no_enum_update = 0
         self.set_tooltip()
 
+    #---------------------------------------------------------------------------
+    #  Returns the QComboBox used for the editor control:
+    #---------------------------------------------------------------------------
+
+    def create_combo_box(self):
+        """ Returns the QComboBox used for the editor control.
+        """
+        control = QtGui.QComboBox()
+        control.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
+        control.setSizePolicy(QtGui.QSizePolicy.Maximum,
+                              QtGui.QSizePolicy.Fixed)
+        return control
+
     #---------------------------------------------------------------------------
     #  Handles the user selecting a new value from the combo box:
     #---------------------------------------------------------------------------
@@ -205,12 +218,13 @@ class SimpleEditor ( BaseEditor ):
     def update_object (self, text):
         """ Handles the user selecting a new value from the combo box.
         """
-        self._no_enum_update += 1
-        try:
-            self.value = self.mapping[unicode(text)]
-        except:
-            pass
-        self._no_enum_update -= 1
+        if self._no_enum_update == 0:
+            self._no_enum_update += 1
+            try:
+                self.value = self.mapping[unicode(text)]
+            except:
+                pass
+            self._no_enum_update -= 1
 
     #---------------------------------------------------------------------------
     #  Handles the user typing text into the combo box text entry field:
@@ -244,10 +258,11 @@ class SimpleEditor ( BaseEditor ):
             editor.
         """
         if self._no_enum_update == 0:
+            self._no_enum_update += 1
             if self.factory.evaluate is None:
                 try:
-                    idx = self.control.findText(self.inverse_mapping[self.value])
-                    self.control.setCurrentIndex(idx)
+                    index = self.names.index(self.inverse_mapping[self.value])
+                    self.control.setCurrentIndex(index)
                 except:
                     self.control.setCurrentIndex(-1)
             else:
@@ -255,6 +270,7 @@ class SimpleEditor ( BaseEditor ):
                     self.control.setEditText(self.str_value)
                 except:
                     self.control.setEditText('')
+            self._no_enum_update -= 1
         
     #---------------------------------------------------------------------------
     #  Handles an error that occurs while setting the object's trait value:
@@ -284,11 +300,13 @@ class SimpleEditor ( BaseEditor ):
         """ Rebuilds the contents of the editor whenever the original factory
             object's **values** trait changes.
         """
+        self.control.blockSignals(True)
         self.control.clear()
         self.control.addItems(self.names)
+        self.control.blockSignals(False)
 
         self.update_editor()
-                                      
+
 #-------------------------------------------------------------------------------
 #  'RadioEditor' class:
 #-------------------------------------------------------------------------------
@@ -297,6 +315,10 @@ class RadioEditor ( BaseEditor ):
     """ Enumeration editor, used for the "custom" style, that displays radio
         buttons.
     """
+
+    # Is the button layout row-major or column-major?
+    row_major = Bool( False )
+
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
     #  widget:
@@ -308,12 +330,13 @@ class RadioEditor ( BaseEditor ):
         """
         super( RadioEditor, self ).init( parent )
 
-        # The control is a grid layout.
-        self.control = QtGui.QGridLayout()
+        self.control = QtGui.QWidget()
+        layout = QtGui.QGridLayout(self.control)
+        layout.setMargin(0)
 
         self._mapper = QtCore.QSignalMapper()
-        QtCore.QObject.connect(self._mapper,
-                QtCore.SIGNAL('mapped(QWidget *)'), self.update_object)
+        QtCore.QObject.connect(self._mapper, QtCore.SIGNAL('mapped(int)'),
+                               self.update_object)
 
         self.rebuild_editor()
    
@@ -321,11 +344,11 @@ class RadioEditor ( BaseEditor ):
     #  Handles the user clicking one of the 'custom' radio buttons:
     #---------------------------------------------------------------------------
     
-    def update_object(self, rb):
+    def update_object ( self, index ):
         """ Handles the user clicking one of the custom radio buttons.
         """
         try:
-            self.value = rb.value
+            self.value = self.mapping[self.names[index]]
         except:
             pass
         
@@ -337,9 +360,10 @@ class RadioEditor ( BaseEditor ):
         """ Updates the editor when the object trait changes externally to the 
             editor.
         """
+        layout = self.control.layout()
         value = self.value
-        for i in range(self.control.count()):
-            rb = self.control.itemAt(i).widget()
+        for i in range(layout.count()):
+            rb = layout.itemAt(i).widget()
             rb.setChecked(rb.value == value)
 
     #---------------------------------------------------------------------------
@@ -363,35 +387,47 @@ class RadioEditor ( BaseEditor ):
         n       = len( names )
         cols    = self.factory.cols
         rows    = (n + cols - 1) / cols
-        incr    = [ n / cols ] * cols
-        rem     = n % cols
-        for i in range( cols ):
-            incr[i] += (rem > i)
-        incr[-1] = -(reduce( lambda x, y: x + y, incr[:-1], 0 ) - 1)
+        if self.row_major:
+            incr = [ 1 ] * cols
+        else:
+            incr = [ n / cols ] * cols
+            rem  = n % cols
+            for i in range( cols ):
+                incr[i] += (rem > i)
+            incr[-1] = -(reduce( lambda x, y: x + y, incr[:-1], 0 ) - 1)
 
         # Add the set of all possible choices:
+        layout = self.control.layout()
         index = 0
-
         for i in range( rows ):
             for j in range( cols ):
                 if n > 0:
-                    name = label = names[ index ]
-                    label = self.string_value( label, capitalize )
-                    rb = QtGui.QRadioButton(label)
+                    name = names[index]
+                    rb = self.create_button(name)
                     rb.value = mapping[name]
 
                     rb.setChecked(name == cur_name)
 
                     QtCore.QObject.connect(rb, QtCore.SIGNAL('clicked()'),
-                            self._mapper, QtCore.SLOT('map()'))
-                    self._mapper.setMapping(rb, rb)
+                                           self._mapper, QtCore.SLOT('map()'))
+                    self._mapper.setMapping(rb, index)
 
                     self.set_tooltip(rb)
-                    self.control.addWidget(rb, i, j)
+                    layout.addWidget(rb, i, j)
 
                     index += incr[j]
                     n -= 1
 
+    #---------------------------------------------------------------------------
+    #  Returns the QAbstractButton used for the radio button:
+    #---------------------------------------------------------------------------
+
+    def create_button(self, name):
+        """ Returns the QAbstractButton used for the radio button.
+        """
+        label = self.string_value(name, capitalize)
+        return QtGui.QRadioButton(label)
+
 #-------------------------------------------------------------------------------
 #  'ListEditor' class:
 #-------------------------------------------------------------------------------
@@ -470,7 +506,11 @@ class ListEditor ( BaseEditor ):
         """ Rebuilds the contents of the editor whenever the original factory
             object's **values** trait changes.
         """
+        
+        self.control.blockSignals(True)
         self.control.clear()
-
         for name in self.names:
             self.control.addItem(name)
+        self.control.blockSignals(False)
+        
+        self.update_editor()
