@@ -82,9 +82,9 @@ class MakeMenu:
             self.menu = menu = QtGui.QMenu()
             self.parse( menu, -1 )
         else:
-            self.menu = menu = wx.MenuBar()
+            self.menu = menu = QtGui.QMenuBar()
             self.parse( menu, -1 )
-            window.SetMenuBar( menu )
+            window.setMenuBar( menu )
 
     #---------------------------------------------------------------------------
     #  Recursively parses menu items from the description:
@@ -136,13 +136,13 @@ class MakeMenu:
                         handler = self.indirect
                     else:
                         try:
-                            exec ('def handler(event,self=self.owner):\n %s\n' %
+                            exec ('def handler(self=self.owner):\n %s\n' %
                                   handler)
                         except:
                            handler = null_handler
                 else:
                     try:
-                        exec 'def handler(event,self=self.owner):\n%s\n' % (
+                        exec 'def handler(self=self.owner):\n%s\n' % (
                             self.get_body( indented ), ) in globals()
                     except:
                         handler = null_handler
