@@ -43,11 +43,36 @@ from constants \
 from helper \
     import IconButton
 
+#-------------------------------------------------------------------------------
+#  'BaseRangeEditor' class:
+#-------------------------------------------------------------------------------
+
+class BaseRangeEditor ( Editor ):
+    """ The base class for Range editors. Using an evaluate trait, if specified,
+        when assigning numbers the object trait.
+    """
+    
+    #---------------------------------------------------------------------------
+    #  Trait definitions:
+    #---------------------------------------------------------------------------
+
+    # Function to evaluate floats/ints
+    evaluate = Any
+
+    #---------------------------------------------------------------------------
+    #  Sets the associated object trait's value:
+    #---------------------------------------------------------------------------
+    
+    def _set_value ( self, value ):
+        if self.evaluate is not None:
+            value = self.evaluate( value )
+        Editor._set_value( self, value )
+
 #-------------------------------------------------------------------------------
 #  'SimpleSliderEditor' class:
 #-------------------------------------------------------------------------------
 
-class SimpleSliderEditor ( Editor ):
+class SimpleSliderEditor ( BaseRangeEditor ):
     """ Simple style of range editor that displays a slider and a text field.
 
     The user can set a value either by moving the slider or by typing a value
@@ -67,9 +92,6 @@ class SimpleSliderEditor ( Editor ):
     # Formatting string used to format value and labels
     format = Str
 
-    # Function to evaluate textual user input
-    evaluate = Any
-
     #---------------------------------------------------------------------------
     #  Finishes initializing the editor by creating the underlying toolkit
     #  widget:
@@ -91,14 +113,12 @@ class SimpleSliderEditor ( Editor ):
         self.evaluate = factory.evaluate
         self.sync_value( factory.evaluate_name, 'evaluate', 'from' )
 
-        if factory.label_width > 0:
-            size = wx.Size( factory.label_width, 20 )
-
         self.sync_value( factory.low_name,  'low',  'from' )
         self.sync_value( factory.high_name, 'high', 'from' )
 
-        # The control is a horizontal layout.
-        self.control = panel = QtGui.QHBoxLayout()
+        self.control = QtGui.QWidget()
+        panel = QtGui.QHBoxLayout(self.control)
+        panel.setMargin(0)
 
         fvalue = self.value
 
@@ -112,10 +132,13 @@ class SimpleSliderEditor ( Editor ):
         ivalue = self._convert_to_slider(fvalue)
 
         self._label_lo = QtGui.QLabel()
-        self._label_lo.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+        self._label_lo.setAlignment(QtCore.Qt.AlignRight | 
+                                    QtCore.Qt.AlignVCenter)
+        if factory.label_width > 0:
+            self._label_lo.setMinimumWidth(factory.label_width)
         panel.addWidget(self._label_lo)
 
-        panel.slider = slider = QtGui.QSlider(QtCore.Qt.Horizontal)
+        self.control.slider = slider = QtGui.QSlider(QtCore.Qt.Horizontal)
         slider.setTracking(factory.auto_set)
         slider.setMinimum(0)
         slider.setMaximum(10000)
@@ -128,8 +151,10 @@ class SimpleSliderEditor ( Editor ):
 
         self._label_hi = QtGui.QLabel()
         panel.addWidget(self._label_hi)
+        if factory.label_width > 0:
+            self._label_hi.setMinimumWidth(factory.label_width)
 
-        panel.text = text = QtGui.QLineEdit(fvalue_text)
+        self.control.text = text = QtGui.QLineEdit(fvalue_text)
         QtCore.QObject.connect(text, QtCore.SIGNAL('editingFinished()'),
                 self.update_object_on_enter)
 
@@ -176,7 +201,7 @@ class SimpleSliderEditor ( Editor ):
         """
         try:
             try:
-                value = self.evaluate(unicode(self.control.text.text()).strip())
+                value = eval(unicode(self.control.text.text()).strip())
             except Exception, ex:
                 # The entered something that didn't eval as a number, (e.g.,
                 # 'foo') pretend it didn't happen
@@ -306,7 +331,7 @@ class LogRangeSliderEditor ( SimpleSliderEditor ):
 #  'LargeRangeSliderEditor' class:
 #-------------------------------------------------------------------------------
 
-class LargeRangeSliderEditor ( Editor ):
+class LargeRangeSliderEditor ( BaseRangeEditor ):
     """ A slider editor for large ranges.
 
     The editor displays a slider and a text field. A subset of the total range
@@ -329,9 +354,6 @@ class LargeRangeSliderEditor ( Editor ):
     # High end of displayed range
     cur_high = Float
 
-    # Function to evaluate textual user input
-    evaluate = Any
-
     # Flag indicating that the UI is in the process of being updated
     ui_changing = Bool(False)
 
@@ -362,8 +384,9 @@ class LargeRangeSliderEditor ( Editor ):
 
         self._set_format()
 
-        # The control is a horizontal layout.
-        self.control = panel = QtGui.QHBoxLayout()
+        self.control = QtGui.QWidget()
+        panel = QtGui.QHBoxLayout(self.control)
+        panel.setMargin(0)
 
         fvalue = self.value
 
@@ -380,17 +403,17 @@ class LargeRangeSliderEditor ( Editor ):
             ivalue = low
 
         # Lower limit label:
-        panel.label_lo = label_lo = QtGui.QLabel()
+        self.control.label_lo = label_lo = QtGui.QLabel()
         label_lo.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
         panel.addWidget(label_lo)
 
         # Lower limit button:
-        panel.button_lo = IconButton(QtGui.QStyle.SP_ArrowLeft,
+        self.control.button_lo = IconButton(QtGui.QStyle.SP_ArrowLeft,
                 self.reduce_range)
-        panel.addWidget(panel.button_lo)
+        panel.addWidget(self.control.button_lo)
 
         # Slider:
-        panel.slider = slider = QtGui.QSlider(QtCore.Qt.Horizontal)
+        self.control.slider = slider = QtGui.QSlider(QtCore.Qt.Horizontal)
         slider.setTracking(factory.auto_set)
         slider.setMinimum(0)
         slider.setMaximum(10000)
@@ -402,16 +425,16 @@ class LargeRangeSliderEditor ( Editor ):
         panel.addWidget(slider)
 
         # Upper limit button:
-        panel.button_hi = IconButton(QtGui.QStyle.SP_ArrowRight,
+        self.control.button_hi = IconButton(QtGui.QStyle.SP_ArrowRight,
                 self.increase_range)
-        panel.addWidget(panel.button_hi)
+        panel.addWidget(self.control.button_hi)
 
         # Upper limit label:
-        panel.label_hi = label_hi = QtGui.QLabel()
+        self.control.label_hi = label_hi = QtGui.QLabel()
         panel.addWidget(label_hi)
 
         # Text entry:
-        panel.text = text = QtGui.QLineEdit(fvalue_text)
+        self.control.text = text = QtGui.QLineEdit(fvalue_text)
         QtCore.QObject.connect(text, QtCore.SIGNAL('editingFinished()'),
                 self.update_object_on_enter)
 
@@ -456,7 +479,7 @@ class LargeRangeSliderEditor ( Editor ):
         """ Handles the user pressing the Enter key in the text field.
         """
         try:
-            self.value = self.evaluate(unicode(self.control.text.text()).strip())
+            self.value = eval(unicode(self.control.text.text()).strip())
         except TraitError, excp:
             pass
 
@@ -613,7 +636,7 @@ class LargeRangeSliderEditor ( Editor ):
 #  'SimpleSpinEditor' class:
 #-------------------------------------------------------------------------------
 
-class SimpleSpinEditor ( Editor ):
+class SimpleSpinEditor ( BaseRangeEditor ):
     """ A simple style of range editor that displays a spin box control.
     """
     #---------------------------------------------------------------------------
@@ -717,6 +740,27 @@ class RangeTextEditor ( TextEditor ):
     value that is outside the allowed range, the background of the field
     changes color to indicate an error.
     """
+
+    #---------------------------------------------------------------------------
+    #  Trait definitions:
+    #---------------------------------------------------------------------------
+
+    # Function to evaluate floats/ints
+    evaluate = Any
+
+    #---------------------------------------------------------------------------
+    #  Finishes initializing the editor by creating the underlying toolkit
+    #  widget:
+    #---------------------------------------------------------------------------
+        
+    def init ( self, parent ):
+        """ Finishes initializing the editor by creating the underlying toolkit
+            widget.
+        """
+        TextEditor.init( self, parent )
+        self.evaluate = self.factory.evaluate
+        self.sync_value( self.factory.evaluate_name, 'evaluate', 'from' )
+    
     #---------------------------------------------------------------------------
     #  Handles the user entering input data in the edit control:
     #---------------------------------------------------------------------------
@@ -725,7 +769,10 @@ class RangeTextEditor ( TextEditor ):
         """ Handles the user entering input data in the edit control.
         """
         try:
-            self.value = eval(unicode(self.control.text()))
+            value = eval(unicode(self.control.text()))
+            if self.evaluate is not None:
+                value = self.evaluate(value)
+            self.value = value
             col = OKColor
         except:
             col = ErrorColor
