@@ -19,12 +19,6 @@
     of objects, etc).
 """
 
-# FIXME: The following features present in the wx implementation are not 
-# currently implemented:
-#     * right_clicked and right_dclicked events
-#     * drag and drop support
-#     * editable labels (column headers)
-
 #-------------------------------------------------------------------------------
 #  Imports:
 #-------------------------------------------------------------------------------
@@ -110,10 +104,9 @@ class TabularEditor(Editor):
     image = Image
 
     #---------------------------------------------------------------------------
-    #  Finishes initializing the editor by creating the underlying toolkit
-    #  widget:
+    #  Editor interface:
     #---------------------------------------------------------------------------
-
+        
     def init (self, parent):
         """ Finishes initializing the editor by creating the underlying toolkit
             widget.
@@ -168,23 +161,23 @@ class TabularEditor(Editor):
         # Make sure we listen for 'items' changes as well as complete list
         # replacements:
         try:
-            self.context_object.on_trait_change(self.update_editor,
-                                self.extended_name + '_items', dispatch = 'ui')
+            self.context_object.on_trait_change(
+                self.update_editor, self.extended_name+'_items', dispatch='ui')
         except:
             pass
 
         # If the user has requested automatic update, attempt to set up the
         # appropriate listeners:
         if factory.auto_update:
-            self.context_object.on_trait_change(self.update_editor,
-                                self.extended_name + '.-', dispatch='ui')
+            self.context_object.on_trait_change(
+                self.refresh_editor, self.extended_name + '.-', dispatch='ui')
 
-        # Create the mapping from user supplied images to wx.ImageList indices:
+        # Create the mapping from user supplied images to QImages:
         for image_resource in factory.images:
             self._add_image(image_resource)
 
         # Refresh the editor whenever the adapter changes:
-        self.on_trait_change(self.update_editor, 'adapter.+update',
+        self.on_trait_change(self.refresh_editor, 'adapter.+update',
                              dispatch='ui')
 
         # Rebuild the editor columns and headers whenever the adapter's
@@ -192,35 +185,59 @@ class TabularEditor(Editor):
         self.on_trait_change(self.update_editor, 'adapter.columns', 
                              dispatch='ui')
 
-    #---------------------------------------------------------------------------
-    #  Disposes of the contents of an editor:
-    #---------------------------------------------------------------------------
-
     def dispose (self):
         """ Disposes of the contents of an editor.
         """
-        self.context_object.on_trait_change(self.update_editor,
-                                  self.extended_name + '_items', remove=True)
+        self.context_object.on_trait_change(
+            self.update_editor, self.extended_name + '_items', remove=True)
 
         if self.factory.auto_update:
-            self.context_object.on_trait_change(self.update_editor,
-                                self.extended_name + '.-', remove=True)
+            self.context_object.on_trait_change(
+                self.refresh_editor, self.extended_name + '.-', remove=True)
 
-        self.on_trait_change(self.update_editor, 'adapter.+update', remove=True)
+        self.on_trait_change(self.refresh_editor, 'adapter.+update', 
+                             remove=True)
         self.on_trait_change(self.update_editor, 'adapter.columns',
                              remove=True)
 
         super(TabularEditor, self).dispose()
 
-    #---------------------------------------------------------------------------
-    #  Updates the editor when the object trait changes external to the editor:
-    #---------------------------------------------------------------------------
-
     def update_editor(self):
         """ Updates the editor when the object trait changes externally to the
             editor.
         """
-        self.model.reset()
+        if not self._no_update:
+            self.model.reset()
+
+    #---------------------------------------------------------------------------
+    #  TabularEditor interface:
+    #---------------------------------------------------------------------------
+
+    def refresh_editor(self):
+        """ Requests the table view to redraw itself.
+        """
+        self.control.viewport().update()
+
+    def callx(self, func, *args, **kw):
+        """ Call a function without allowing the editor to update.
+        """
+        old = self._no_update
+        self._no_update = True
+        try:
+            func(*args, **kw)
+        finally:
+            self._no_update = old
+
+    def setx(self, **keywords):
+        """ Set one or more attributes without allowing the editor to update.
+        """
+        old = self._no_notify
+        self._no_notify = True
+        try:
+            for name, value in keywords.items():
+                setattr(self, name, value)
+        finally:
+            self._no_notify = old
 
     #---------------------------------------------------------------------------
     #  UI preference save/restore interface:
@@ -239,8 +256,8 @@ class TabularEditor(Editor):
     def save_prefs(self):
         """ Returns any user preference information associated with the editor.
         """
-        widths = [ self.control.columnWidth(column) for column
-                   in xrange(len(self.adapter.columns)) ]
+        widths = [ self.control.columnWidth(column) 
+                   for column in xrange(len(self.adapter.columns)) ]
         return { 'cached_widths': widths }
 
     #---------------------------------------------------------------------------
@@ -288,17 +305,19 @@ class TabularEditor(Editor):
     def _selected_changed(self, new):
         if not self._no_update:
             try:
-                self._selected_row_changed(self, self.value.index(new))
+                selected_row = self.value.index(new)
             except:
                 pass
+            else:
+                self._selected_row_changed(selected_row)
 
     def _selected_row_changed(self, selected_row):
         if not self._no_update:
             smodel = self.control.selectionModel()
-            if selected is None:
+            if selected_row == -1:
                 smodel.clearSelection()
             else:
-                smodel.select(self.model.index(row, 0), 
+                smodel.select(self.model.index(selected_row, 0), 
                               QtGui.QItemSelectionModel.ClearAndSelect |
                               QtGui.QItemSelectionModel.Rows)
 
@@ -307,19 +326,21 @@ class TabularEditor(Editor):
             values = self.value
             try:
                 rows = [ values.index(i) for i in new]
-                self._multi_selected_rows_changed(row)
             except:
                 pass
+            else:
+                self._multi_selected_rows_changed(rows)
 
     def _multi_selected_items_changed(self, event):
         values = self.values
         try:
             added = [ values.index(item) for item in event.added ]
             removed = [ values.index(item) for item in event.removed ]
-            new_event = TraitListEvent(0, added, removed)
-            self._multi_selected_rows_items_changed(new_event)
         except:
             pass
+        else:
+            list_event = TraitListEvent(0, added, removed)
+            self._multi_selected_rows_items_changed(list_event)
 
     def _multi_selected_rows_changed(self, selected_rows):
         if not self._no_update:
@@ -346,7 +367,8 @@ class TabularEditor(Editor):
     def _on_activate(self, index):
         """ Handle a cell being activated.
         """
-        self._mouse_click(index, 'activated')
+        self.activated_row = row = index.row()
+        self.activated = self.adapter.get_item(self.object, self.name, row)
 
     def _on_click(self, index):
         """ Handle a cell being clicked.
@@ -429,14 +451,13 @@ class TabularEditorEvent(HasStrictTraits):
 class _ItemDelegate(QtGui.QStyledItemDelegate):
     """ A QStyledItemDelegate which draws its owns gridlines so that we can 
         choose to draw only the horizontal or only the vertical gridlines if 
-        necessary.
+        appropriate.
     """
 
     def __init__(self, table_view):
         """ Store which grid lines to draw.
         """
-        QtGui.QStyledItemDelegate.__init__(self)
-        self._table_view = table_view
+        QtGui.QStyledItemDelegate.__init__(self, table_view)
         self._horizontal_lines = table_view._editor.factory.horizontal_lines
         self._vertical_lines = table_view._editor.factory.vertical_lines
 
@@ -447,19 +468,15 @@ class _ItemDelegate(QtGui.QStyledItemDelegate):
         painter.save()
 
         # FIXME: 'styleHint' is returning bogus (negative) values. Why?
-        #style = self._table_view.style()
-        #color = style.styleHint(QtGui.QStyle.SH_Table_GridLineColor, option,
-        #                        self._table_view)
+        #style = QtGui.QApplication.instance().style()
+        #color = style.styleHint(QtGui.QStyle.SH_Table_GridLineColor, option)
         #painter.setPen(QtGui.QColor(color))
         painter.setPen(option.palette.color(QtGui.QPalette.Dark))
 
         if self._horizontal_lines:
-            painter.drawLine(option.rect.topLeft(), option.rect.topRight())
-            if index.row() == index.model().rowCount(index)-1:
-                painter.drawLine(option.rect.bottomLeft(), 
-                                 option.rect.bottomRight())
+            painter.drawLine(option.rect.bottomLeft(),option.rect.bottomRight())
         if self._vertical_lines:
-            painter.drawLine(option.rect.topLeft(), option.rect.bottomLeft())
+            painter.drawLine(option.rect.topRight(), option.rect.bottomRight())
 
         painter.restore()
 
@@ -471,13 +488,26 @@ class _TableView(QtGui.QTableView):
         """ Initialise the object.
         """
         QtGui.QTableView.__init__(self)
-
+        
+        self._initial_size = False
         self._editor = editor
         self.setModel(editor.model)
         factory = editor.factory
         
         # Configure the row headings
-        self.verticalHeader().hide()
+        vheader = self.verticalHeader()
+        vheader.hide()
+
+        # Set a default height for rows. Although setting the resize mode to
+        # ResizeToContents would provide the best sizes, this is far too
+        # expensive when the TabularEditor has a large amount of data. Instead,
+        # we make a reasonable guess based on the minimum size hint and the font
+        # of the first row.
+        size = vheader.minimumSectionSize()
+        font = editor.adapter.get_font(editor.object, editor.name, 0)
+        if font is not None:
+            size = max(size, QtGui.QFontMetrics(QtGui.QFont(font)).height())
+        vheader.setDefaultSectionSize(size)
 
         # Configure the column headings.
         hheader = self.horizontalHeader()
@@ -486,7 +516,6 @@ class _TableView(QtGui.QTableView):
             hheader.setHighlightSections(False)
         else:
             hheader.hide()
-        self.resizeColumnsToContents()
 
         # Turn off the grid lines--we'll draw our own
         self.setShowGrid(False)
@@ -500,20 +529,126 @@ class _TableView(QtGui.QTableView):
             mode = QtGui.QAbstractItemView.SingleSelection
         self.setSelectionMode(mode)
 
-        # Set column widths according to the adapter's requested width
-        for column in xrange(len(editor.adapter.columns)):
-            width = editor.adapter.get_width(editor.object, editor.name, column)
-            if width != -1:
-                self.setColumnWidth(column, width)
+        # Configure drag and drop behavior
+        self.setDragEnabled(True)
+        self.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
+        self.setDropIndicatorShown(True)
+
+    def keyPressEvent(self, event):
+        """ Reimplemented to support edit, insert, and delete by keyboard.
+        """
+        editor = self._editor
+        factory = editor.factory
+
+        # Note that setting 'EditKeyPressed' as an edit trigger does not work on
+        # most platforms, which is why we do this here.
+        if (event.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return) and
+            self.state() != QtGui.QAbstractItemView.EditingState and 
+            factory.editable and 'edit' in factory.operations):
+            if factory.multi_select:
+                rows = editor.multi_selected_rows
+                row = rows[0] if len(rows) == 1 else -1
+            else:
+                row = editor.selected_row
+            
+            if row != -1:
+                event.accept()
+                self.edit(editor.model.index(row, 0))
+
+        elif (event.key() in (QtCore.Qt.Key_Backspace, QtCore.Qt.Key_Delete) and
+              factory.editable and 'delete' in factory.operations):
+            event.accept()
+
+            if factory.multi_select:
+                for row in reversed(sorted(editor.multi_selected_rows)):
+                    editor.model.removeRow(row)
+            elif editor.selected_row != -1:
+                editor.model.removeRow(editor.selected_row)
+
+        elif (event.key() == QtCore.Qt.Key_Insert and 
+              factory.editable and 'insert' in factory.operations):
+            event.accept()
+
+            if factory.multi_select:
+                rows = sorted(editor.multi_selected_rows)
+                row = rows[0] if len(rows) else -1
+            else:
+                row = editor.selected_row
+            if row == -1:
+                row = editor.adapter.len(editor.object, editor.name)
+            editor.model.insertRow(row)
+            self.setCurrentIndex(editor.model.index(row, 0))
+
+        else:
+            QtGui.QTableView.keyPressEvent(self, event)
 
     def sizeHint(self):
-        """ Reimplemented to define an appropriate size hint.
+        """ Reimplemented to define a reasonable size hint.
         """
         sh = QtGui.QTableView.sizeHint(self)
         
-        w = 0
+        width = 0
         for column in xrange(len(self._editor.adapter.columns)):
-            w += self.sizeHintForColumn(column)
-        sh.setWidth(w)
+            width += self.sizeHintForColumn(column)
+        sh.setWidth(width)
 
         return sh
+
+    def resizeEvent(self, event):
+        """ Reimplemented to size the table columns when the size of the table
+            changes. Because the layout algorithm requires that the available
+            space be known, we have to wait until the UI that contains this
+            table gives it its initial size.
+        """
+        QtGui.QTableView.resizeEvent(self, event)
+
+        parent = self.parent()
+        if (not self._initial_size and parent and
+            (self.isVisible() or isinstance(parent, QtGui.QMainWindow))):
+            self._initial_size = True
+            self.resizeColumnsToContents()
+
+    def sizeHintForColumn(self, column):
+        """ Reimplemented to support absolute width specification via
+            TabularAdapters and to avoid scanning all data to determine the size
+            hint. (TabularEditor, unlike TableEditor, is expected to handle very
+            large data sets.)
+        """
+        editor = self._editor
+
+        width = editor.adapter.get_width(editor.object, editor.name, column)
+        if width > 1:
+            return width
+        else:
+            return self.horizontalHeader().sectionSizeHint(column)
+
+    def resizeColumnsToContents(self):
+        """ Reimplemented to support proportional column width specifications.
+            For information about the layout algorithm, see
+            https://svn.enthought.com/enthought/wiki/Traits_3_0_tabular_editor.
+        """
+        editor = self._editor
+        available_space = self.viewport().width()
+        hheader = self.horizontalHeader()
+        
+        # Assign sizes for columns with absolute size requests
+        percent_vals, percent_cols = [], []
+        for column in xrange(len(editor.adapter.columns)):
+            width = editor.adapter.get_width(editor.object, editor.name, column)
+            if width > 1:
+                available_space -= width
+                hheader.resizeSection(column, width)
+            else:
+                if width <= 0:
+                    width = 0.1
+                percent_vals.append(width)
+                percent_cols.append(column)
+
+        # Now use the remaining space for columns with proportional or no width
+        # requests.
+        percent_total = sum(percent_vals)
+        for i, column in enumerate(percent_cols):
+            percent = percent_vals[i] / percent_total
+            width = max(30, int(percent * available_space))
+            hheader.resizeSection(column, width)
+            
