@@ -17,6 +17,10 @@ the PyQt user interface toolkit.
 #  Imports:
 #-------------------------------------------------------------------------------
 
+# Make sure that importing from this backend is OK:
+from enthought.traits.ui.toolkit import assert_toolkit_import
+assert_toolkit_import('qt4')
+
 from PyQt4 import QtCore, QtGui
 
 # Check the version numbers are late enough:
@@ -28,12 +32,10 @@ import sys
 if QtGui.QApplication.startingUp():
     _app = QtGui.QApplication(sys.argv)
 
-# Make sure that SIGINTs actually stop the application event loop:
+# Make sure that SIGINTs actually stop the application event loop (Qt sometimes
+# swallows KeyboardInterrupt exceptions):
 import signal
-def _handler(signum, frame):
-    QtGui.QApplication.instance().quit()
-    sys.exit(signum)
-signal.signal(signal.SIGINT, _handler)
+signal.signal(signal.SIGINT, signal.SIG_DFL)
 
 from enthought.traits.trait_notifiers import set_ui_handler
 
@@ -60,7 +62,8 @@ class _CallAfter(QtCore.QObject):
     _calls_mutex = QtCore.QMutex()
 
     def __init__(self, handler, *args):
-        """ Initialise the call. """
+        """ Initialise the call.
+        """
         QtCore.QObject.__init__(self)
 
         # Save the details of the call.
@@ -82,7 +85,8 @@ class _CallAfter(QtCore.QObject):
         QtGui.QApplication.instance().postEvent(self, event)
 
     def event(self, event):
-        """ QObject event handler. """
+        """ QObject event handler.
+        """
         if event.type() == _QT_TRAITS_EVENT:
             # Invoke the handler
             self._handler(*self._args)
@@ -283,7 +287,13 @@ class GUIToolkit ( Toolkit ):
             y = int( y )
 
         # Position and size the window as requested:
-        window.setGeometry( max( 0, x ), max( 0, y ), width, height )
+        layout = window.layout()
+        if layout.sizeConstraint() == QtGui.QLayout.SetFixedSize:
+            layout.setSizeConstraint( QtGui.QLayout.SetDefaultConstraint )
+            window.move( max( 0, x ), max( 0, y ) )
+            window.setFixedSize( QtCore.QSize ( width, height ) )
+        else:
+            window.setGeometry( max( 0, x ), max( 0, y ), width, height )
 
     #---------------------------------------------------------------------------
     #  Shows a 'Help' window for a specified UI and control:
@@ -312,18 +322,10 @@ class GUIToolkit ( Toolkit ):
     def rebuild_ui ( self, ui ):
         """ Rebuilds a UI after a change to the content of the UI.
         """
-        parent = size = None
         if ui.control is not None:
-            size   = ui.control.GetSize()
-            parent = ui.control._parent
             ui.recycle()
             ui.info.ui = ui
-        ui.rebuild( ui, parent )
-        if parent is not None:
-            ui.control.SetSize( size )
-            sizer = parent.GetSizer()
-            if sizer is not None:
-                sizer.Add( ui.control, 1, wx.EXPAND )
+        ui.rebuild( ui, ui.parent )
 
     #---------------------------------------------------------------------------
     #  Sets the title for the UI window:
@@ -358,7 +360,7 @@ class GUIToolkit ( Toolkit ):
 
     #---------------------------------------------------------------------------
     #  Hooks all specified events for all controls in a ui so that they can be
-    #  routed to the corrent event handler:
+    #  routed to the correct event handler:
     #---------------------------------------------------------------------------
 
     def hook_events ( self, ui, control, events = None, handler = None ):
@@ -366,7 +368,7 @@ class GUIToolkit ( Toolkit ):
             can be routed to the correct event handler.
         """
         if events is None:
-            # fixme: Implement drag and drop events ala toolkit.py for wx
+            # FIXME: Implement drag and drop events ala toolkit.py for wx
             return
 
         elif events == 'keys':
@@ -395,21 +397,17 @@ class GUIToolkit ( Toolkit ):
     def destroy_control ( self, control ):
         """ Destroys a specified GUI toolkit control.
         """
-        # Make sure that any dialogs started with 'exec' are terminated,
-        # otherwise their local event loops will continue to run.
-        if isinstance(control, QtGui.QDialog):
-            # Block signals to prevent any 'finished()' signals being emitted.
-            # At this point, all cleanup is assumed to be done.
-            control.blockSignals(True)
-            QtGui.QDialog.done(control, 0)
-            control.blockSignals(False)
+        # Block signals to prevent any editors from being updated (the control
+        # will not be deleted immediately).
+        control.blockSignals(True)
 
-        # This may be called from within the finished() signal handler
-        # so we need to do the delete after the handler has returned.
+        # This may be called from within the finished() signal handler so we
+        # need to do the delete after the handler has returned.
+        control.hide()
         control.deleteLater()
 
-        # PyQt v4.3.1 and earlier deleteLater() didn't transfer ownership
-        # to C++ which turns out to be important.
+        # PyQt v4.3.1 and earlier deleteLater() didn't transfer ownership to
+        # C++, which is necessary for the QObject system to garbage collect it.
         if QtCore.PYQT_VERSION < 0x040302:
             import sip
             sip.transferto(control, None)
@@ -425,7 +423,9 @@ class GUIToolkit ( Toolkit ):
         for w in control.children():
             # Only destroy widgets.
             if isinstance(w, QtGui.QWidget):
-                w.setParent(None)
+                # This may be called from within the finished() signal handler
+                # so we need to do the delete after the handler has returned.
+                w.deleteLater()
 
     #---------------------------------------------------------------------------
     #  Returns a ( width, height ) tuple containing the size of a specified
