@@ -32,14 +32,10 @@ from enthought.traits.trait_base \
 from enthought.traits.ui.api \
     import TreeNode, ObjectTreeNode, MultiTreeNode
      
-# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
-# compatibility. The class has been moved to the 
-# enthought.traits.ui.editors.tree_editor file.
-from enthought.traits.ui.editors.tree_editor \
-    import ToolkitEditorFactory
-    
 from enthought.traits.ui.undo \
     import ListUndoItem
+    
+from enthought.traits.ui.tree_node import ITreeNodeAdapterBridge    
 
 from enthought.traits.ui.menu \
     import Menu, Action, Separator
@@ -117,7 +113,7 @@ class SimpleEditor ( Editor ):
         self._editor = None
 
         if factory.editable:
-
+            
             # Check to see if the tree view is based on a shared trait editor:
             if factory.shared_editor:
                 factory_editor = factory.editor
@@ -127,9 +123,10 @@ class SimpleEditor ( Editor ):
 
                     # Remember which editor has the trait editor in the factory:
                     factory._editor = self
-
+                    
                     # Create the trait editor panel:
-                    self.control = QtGui.QWidget(panel)
+                    self.control = QtGui.QWidget()
+                    parent.addWidget(self.control)
                     self.control._node_ui = self.control._editor_nid = None
 
                     # Check to see if there are any existing editors that are
@@ -179,6 +176,8 @@ class SimpleEditor ( Editor ):
                     orient = QtCore.Qt.Vertical
 
                 self.control = splitter = QtGui.QSplitter(orient)
+                splitter.setSizePolicy(QtGui.QSizePolicy.Expanding,
+                                       QtGui.QSizePolicy.Expanding)
                 splitter.addWidget(self._tree)
                 splitter.addWidget(sa)
         else:
@@ -284,7 +283,7 @@ class SimpleEditor ( Editor ):
                 nid.setIcon(0, self._get_icon(node, object))
                 nid.setToolTip(0, node.get_tooltip(object))
 
-            self._map[ id( object ) ] = [ ( node.children, nid ) ]
+            self._map[ id( object ) ] = [ ( node.get_children_id(object), nid ) ]
             self._add_listeners( node, object )
             self._set_node_data( nid, ( False, node, object) )
             if self.factory.hide_root or self._has_children( node, object ):
@@ -320,7 +319,7 @@ class SimpleEditor ( Editor ):
         has_children = self._has_children(node, object)
         self._set_node_data( cnid, ( False, node, object ) )
         self._map.setdefault( id( object ), [] ).append(
-            ( node.children, cnid ) )
+            ( node.get_children_id(object), cnid ) )
         self._add_listeners( node, object )
 
         # Automatically expand the new node (if requested):
@@ -553,7 +552,7 @@ class SimpleEditor ( Editor ):
 
         # If none found, give up:
         if len( nodes ) == 0:
-            return ( object, None )
+            return ( object, ITreeNodeAdapterBridge(adapter=object) )
 
         # Use all selected nodes that have the same 'node_for' list as the
         # first selected node:
@@ -858,25 +857,36 @@ class SimpleEditor ( Editor ):
         """ Handles a tree node being selected.
         """
         # Get the new selection:
-        sel = self._tree.selectedItems()
+        nids = self._tree.selectedItems()
 
-        if len(sel) > 0:
-            nid = sel[0]
+        selected = []
+        if len(nids) > 0:
+            for n in nids:
+                # If there is a real selection, get the associated object:
+                expanded, node, sel_object = self._get_node_data(n)
+                selected.append(sel_object)
 
-            # If there is a real selection, get the associated object:
-            expanded, node, object = self._get_node_data(nid)
-
-            # Try to inform the node specific handler of the selection:
-            not_handled = node.select(object)
+                # Try to inform the node specific handler of the selection, if
+                # there are multiple selections, we only care about the first
+                # (or maybe the last makes more sense?)
+                if n == nids[0]:
+                    nid = n
+                    object = sel_object
+                    not_handled = node.select(sel_object)
         else:
             nid = None
             object = None
             not_handled = True
 
         # Set the value of the new selection:
-        self._no_update_selected = True
-        self.selected            = object
-        self._no_update_selected = False
+        if self.factory.selection_mode == 'single': 
+            self._no_update_selected = True 
+            self.selected = object 
+            self._no_update_selected = False 
+        else: 
+            self._no_update_selected = True 
+            self.selected = selected 
+            self._no_update_selected = False 
 
         # If no one has been notified of the selection yet, inform the editor's
         # select handler (if any) of the new selection:
@@ -1445,6 +1455,9 @@ class _TreeWidget(QtGui.QTreeWidget):
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
 
+        if editor.factory.selection_mode == 'extended':
+            self.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+
         self.connect(self, QtCore.SIGNAL('itemExpanded(QTreeWidgetItem *)'),
                 editor._on_item_expanded)
         self.connect(self, QtCore.SIGNAL('itemCollapsed(QTreeWidgetItem *)'),
