@@ -260,23 +260,27 @@ class _StickyDialog(QtGui.QDialog):
     def __init__(self, ui, parent):
         """Initialise the dialog."""
 
-        flags = QtCore.Qt.WindowSystemMenuHint
-        if ui.view.resizable:
-            flags |= QtCore.Qt.WindowMinMaxButtonsHint
-
-        QtGui.QDialog.__init__(self, parent, flags)
+        QtGui.QDialog.__init__(self, parent)
 
         # Create the main window so we can add toolbars etc.
-        layout = QtGui.QVBoxLayout()
         self._mw = QtGui.QMainWindow()
+        layout = QtGui.QVBoxLayout()
+        layout.setMargin(0)
         layout.addWidget(self._mw)
         self.setLayout(layout)
 
+        # Set the dialog's window flags and properties.
         if ui.view.resizable:
-            self.setWindowFlags(QtCore.Qt.WindowMinimizeButtonHint |
-                                QtCore.Qt.WindowMaximizeButtonHint)
+            flags = QtCore.Qt.Window
         else:
+            flags = QtCore.Qt.Dialog | QtCore.Qt.WindowSystemMenuHint
             layout.setSizeConstraint(QtGui.QLayout.SetFixedSize)
+        try:
+            flags |= QtCore.Qt.WindowCloseButtonHint
+        except AttributeError:
+            # Either PyQt or Qt is too old.
+            pass
+        self.setWindowFlags(flags)
 
         self._ui = ui
         self._result = None
@@ -411,6 +415,7 @@ class BaseDialog(BasePanel):
         if style == BaseDialog.NONMODAL:
             ui.control.show()
         else:
+            ui.control.setWindowModality(QtCore.Qt.WindowModal)
             ui.control.exec_()
 
     def set_icon(self, icon=None):
@@ -452,11 +457,9 @@ class BaseDialog(BasePanel):
         toolbar = self.ui.view.toolbar
         if toolbar is not None:
             self._last_group = self._last_parent = None
-            # FIXME: A QDialog has no toolbar. This should be fixed, but right 
-            # now we are passing rather than creating a traceback.
-            return 
-            self.control.SetToolBar(
-                toolbar.create_tool_bar( self.control, self ) )
+            qt_toolbar = toolbar.create_tool_bar( self.control, self )
+            qt_toolbar.setMovable( False )
+            self.control._mw.addToolBar( qt_toolbar )
             self._last_group = self._last_parent = None
 
     #---------------------------------------------------------------------------
@@ -466,12 +469,46 @@ class BaseDialog(BasePanel):
     def _add_statusbar ( self ):
         """ Adds a statusbar to the dialog.
         """
-        # FIXME: We should implement this, but right now we are passing rather
-        # than creating a traceback.
-        return
-        statusbar = self.ui.view.statusbar
-        if statusbar is not None:
-            raise NotImplementedError
+        if self.ui.view.statusbar is not None:
+            control = QtGui.QStatusBar()
+            control.setSizeGripEnabled(self.ui.view.resizable)
+            listeners = []
+            for item in self.ui.view.statusbar:
+                # Create the status widget with initial text
+                name = item.name
+                item_control = QtGui.QLabel()
+                item_control.setText(self.ui.get_extended_value(name))
+                
+                # Add the widget to the control with correct size
+                width = abs(item.width)
+                stretch = 0
+                if width <= 1.0:
+                    stretch = int(100 * width)
+                else:
+                    item_control.setMinimumWidth(width)
+                control.addWidget(item_control, stretch)
+                
+                # Set up event listener for updating the status text
+                col = name.find('.')
+                obj = 'object'
+                if col >= 0:
+                    obj = name[:col]
+                    name = name[col+1:]
+                obj = self.ui.context[obj]
+                set_text = self._set_status_text(item_control)
+                obj.on_trait_change(set_text, name, dispatch='ui')
+                listeners.append((obj, set_text, name))
+
+            self.control._mw.setStatusBar(control)
+            self.ui._statusbar = listeners
+
+    def _set_status_text(self, control):
+        """ Helper function for _add_statusbar.
+        """
+        def set_status_text(text):
+            control.setText(text)
+
+        return set_status_text
 
     #---------------------------------------------------------------------------
     #  Adds a menu item to the menu bar being constructed:
