@@ -12,14 +12,17 @@
 """Creates a panel-based PyQt user interface for a specified UI object.
 """
 
+#-------------------------------------------------------------------------------
+#  Imports:
+#-------------------------------------------------------------------------------
 
 import cgi
 import re
 
-from PyQt4 import QtCore, QtGui
+from PyQt4 import QtCore, QtGui, QtWebKit
 
 from enthought.traits.api \
-    import Instance
+    import Any, Instance, Undefined
 
 from enthought.traits.ui.api \
     import Group
@@ -64,21 +67,21 @@ all_digits = re.compile(r'\d+')
 def ui_panel(ui, parent):
     """Creates a panel-based PyQt user interface for a specified UI object.
     """
-    _ui_panel_for(ui, parent, True)
+    _ui_panel_for(ui, parent, False)
 
 
 def ui_subpanel(ui, parent):
     """Creates a subpanel-based PyQt user interface for a specified UI object.
        A subpanel does not allow control buttons (other than those specified in
-       the UI object).
+       the UI object) and does not show headers for view titles.
     """
-    _ui_panel_for(ui, parent, False)
+    _ui_panel_for(ui, parent, True)
 
 
-def _ui_panel_for(ui, parent, buttons):
+def _ui_panel_for(ui, parent, is_subpanel):
     """Creates a panel-based PyQt user interface for a specified UI object.
     """
-    ui.control = control = _Panel(ui, parent, buttons).control
+    ui.control = control = _Panel(ui, parent, is_subpanel).control
 
     control._parent = parent
     control._object = ui.context.get('object')
@@ -101,7 +104,7 @@ class _Panel(BasePanel):
     """PyQt user interface panel for Traits-based user interfaces.
     """
 
-    def __init__(self, ui, parent, allow_buttons):
+    def __init__(self, ui, parent, is_subpanel):
         """Initialise the object.
         """
         self.ui = ui
@@ -118,7 +121,8 @@ class _Panel(BasePanel):
         # Determine if we need any buttons or an 'undo' history.
         buttons = [self.coerce_button(button) for button in view.buttons]
         nr_buttons = len(buttons)
-        has_buttons = (allow_buttons and ((nr_buttons != 1) or (not self.is_button(buttons[0], ''))))
+        has_buttons = (not is_subpanel and (nr_buttons != 1 or
+                                            not self.is_button(buttons[0], '')))
 
         if nr_buttons == 0:
             if view.undo:
@@ -128,7 +132,7 @@ class _Panel(BasePanel):
             if view.help:
                 self.check_button(buttons, HelpButton)
 
-        if allow_buttons and history is None:
+        if not is_subpanel and history is None:
             for button in buttons:
                 if self.is_button(button, 'Undo') or self.is_button(button, 'Revert'):
                     history = ui.history = UndoHistory()
@@ -137,12 +141,11 @@ class _Panel(BasePanel):
         # Create the panel.
         self.control = panel(ui)
 
-        # Suppress the title if we think it should be superceded by the title
-        # of an "outer" widget (eg. a dock widget).  These tests are extremely
-        # flakey.
+        # Suppress the title if this is a subpanel or if we think it should be
+        # superceded by the title of an "outer" widget (eg. a dock widget).
         title = view.title
-        if ((isinstance(parent, QtGui.QMainWindow) and 
-             not isinstance(parent.parent(), QtGui.QDialog)) or
+        if (is_subpanel or (isinstance(parent, QtGui.QMainWindow) and 
+                            not isinstance(parent.parent(), QtGui.QDialog)) or
             isinstance(parent, QtGui.QTabWidget)):
             title = ""
 
@@ -277,33 +280,17 @@ def _fill_panel(panel, content, ui, item_handler=None):
             if layout is not None:
                 layout.setAlignment(QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
 
-            # Add the content.
-            if isinstance(panel, QtGui.QTabWidget):
-                panel.addTab(new, page_name)
-            else:
-                panel.addItem(new, page_name)
+        else:
+            new = QtGui.QWidget()
+            layout = QtGui.QVBoxLayout(new)
+            layout.setMargin(0)
+            item_handler(item, layout)
 
-        # FIXME: GV: I have commented out the following (wx) code. Has a
-        # result the dialog display under Qt, thought with a wrong
-        # layout. It is better than nothing.
-        #else:
-            # FIXME: Don't yet have an example that exercises this code.
-            # Create the new page as a simple DockControl containing the
-            # specified set of controls:
-            #sizer = wx.BoxSizer( wx.VERTICAL )
-            #panel = QtGui.QWidget(nb)
-            #panel.SetSizer( sizer )
-            #
-            #pages.append( DockControl( name     = page_name,
-            #                           image    = item.image,
-            #                           id       = item.get_id(),
-            #                           style    = item.dock,
-            #                           dockable = DockableViewElement(
-            #                                          ui = ui, element = item ),
-            #                           export   = item.export,
-            #                           control  = panel ) )
-            #item_handler( item, panel, sizer )
-            #panel.GetSizer().Fit( panel )
+        # Add the content.
+        if isinstance(panel, QtGui.QTabWidget):
+            panel.addTab(new, page_name)
+        else:
+            panel.addItem(new, page_name)
 
     panel.setCurrentIndex(active)
 
@@ -349,6 +336,87 @@ def show_help_popup ( event ):
         HTMLHelpWindow( control, html, .25, .13 )
 
 
+class _GroupSplitter(QtGui.QSplitter):
+    """ A splitter for a Traits UI Group with layout='split'.
+    """
+
+    def __init__(self, group):
+        """ Store the group.
+        """
+        QtGui.QSplitter.__init__(self)
+        self._group = group
+        self._initialized = False
+
+    def resizeEvent(self, event):
+        """ Overridden to position the splitter based on the Group when the 
+            application is initializing.
+
+            Because the splitter layout algorithm requires that the available
+            space be known, we have to wait until the UI that contains this
+            splitter gives it its initial size.
+        """
+        QtGui.QSplitter.resizeEvent(self, event)
+
+        parent = self.parent()
+        if (not self._initialized and parent and
+            (self.isVisible() or isinstance(parent, QtGui.QMainWindow))):
+            self._initialized = True
+            self._resize_items()
+
+    def _resize_items(self):
+        """ Size the splitter based on the 'width' or 'height' attributes
+            of the Traits UI view elements.
+        """
+        use_widths = (self.orientation() == QtCore.Qt.Horizontal)
+                
+        # Get the requested size for the items.
+        sizes = []
+        for item in self._group.content:
+            if use_widths:
+                sizes.append(item.width)
+            else:
+                sizes.append(item.height)
+
+        # Find out how much space is available.
+        if use_widths:
+            total = self.width()
+        else:
+            total = self.height()
+
+        # Allocate requested space.
+        avail = total
+        remain = 0
+        for i, sz in enumerate(sizes):
+            if avail <= 0:
+                break
+
+            if sz >= 0:
+                if sz >= 1:
+                    sz = min(sz, avail)
+                else:
+                    sz *= total
+
+                sz = int(sz)
+                sizes[i] = sz
+                avail -= sz
+            else:
+                remain += 1
+
+        # Allocate the remainder to those parts that didn't request a width.
+        if remain > 0:
+            remain = int(avail / remain)
+
+            for i, sz in enumerate(sizes):
+                if sz < 0:
+                    sizes[i] = remain
+
+        # If all requested a width, allocate the remainder to the last item.
+        else:
+            sizes[-1] += avail
+
+        self.setSizes(sizes)
+
+
 class _GroupPanel(object):
     """A sub-panel for a single group of items.  It may be either a layout or a
        widget.
@@ -400,36 +468,46 @@ class _GroupPanel(object):
 
         elif group.layout == 'split':
             # Create the splitter.
-            splitter = QtGui.QSplitter()
-
-            size_policy = splitter.sizePolicy()
-            if group.orientation == 'horizontal':
-                size_policy.setVerticalPolicy(QtGui.QSizePolicy.Expanding)
-            else:
-                size_policy.setHorizontalPolicy(QtGui.QSizePolicy.Expanding)
-            splitter.setSizePolicy(size_policy)
-
+            splitter = _GroupSplitter(group)
+            splitter.setOpaqueResize(False) # Mimic wx backend resize behavior
             if self.direction == QtGui.QBoxLayout.TopToBottom:
                 splitter.setOrientation(QtCore.Qt.Vertical)
 
+            # Make sure the splitter will expand to fill available space
+            policy = splitter.sizePolicy()
+            policy.setHorizontalStretch(50)
+            policy.setVerticalStretch(50)
+            if group.orientation == 'horizontal':
+                policy.setVerticalPolicy(QtGui.QSizePolicy.Expanding)
+            else:
+                policy.setHorizontalPolicy(QtGui.QSizePolicy.Expanding)
+            splitter.setSizePolicy(policy)
+
             if outer is None:
                 outer = splitter
             else:
                 inner.addWidget(splitter)
 
             # Create an editor.
-            self._setup_editor(group,
-                    SplitterGroupEditor(control=outer, splitter=splitter,
-                                        ui=ui))
-
+            editor = SplitterGroupEditor(control=outer, splitter=splitter,ui=ui)
+            self._setup_editor(group, editor)
+                   
             self._add_splitter_items(content, splitter)
 
         elif group.layout in ('tabbed', 'fold'):
+            # Create the TabWidget or ToolBox.
             if group.layout == 'tabbed':
                 sub = QtGui.QTabWidget()
             else:
                 sub = QtGui.QToolBox()
 
+            # Give tab/tool widget stretch factor equivalent to default stretch
+            # factory for a resizeable item. See end of '_add_items'.
+            policy = sub.sizePolicy()
+            policy.setHorizontalStretch(50)
+            policy.setVerticalStretch(50)
+            sub.setSizePolicy(policy)
+
             _fill_panel(sub, content, self.ui, self._add_page_item)
 
             if outer is None:
@@ -438,7 +516,8 @@ class _GroupPanel(object):
                 inner.addWidget(sub)
 
             # Create an editor.
-            self._setup_editor(group, GroupEditor(control=outer))
+            editor = TabbedFoldGroupEditor(container=sub, control=outer, ui=ui)
+            self._setup_editor(group, editor)
 
         else:
             # See if we need to control the visual appearence of the group.
@@ -470,30 +549,13 @@ class _GroupPanel(object):
     def _add_splitter_items(self, content, splitter):
         """Adds a set of groups or items separated by splitter bars.
         """
-        use_widths = (splitter.orientation() == QtCore.Qt.Horizontal)
-        sizes = []
         for item in content:
 
-            # Get a size for the Item or Group
+            # Get a panel for the Item or Group.
             if isinstance(item, Group):
                 panel = _GroupPanel(item, self.ui, suppress_label=True).control
-
-                def max_child_size(item):
-                    if isinstance(item, Group):
-                        return max(map(max_child_size, item.content))
-                    else:
-                        if use_widths:
-                            return item.width
-                        else:
-                            return item.height
-
-                sizes.append(max_child_size(item))
             else:
                 panel = self._add_items([item])
-                if use_widths:
-                    sizes.append(item.width)
-                else:
-                    sizes.append(item.height)
 
             # Add the panel to the splitter.
             if panel is not None:
@@ -510,42 +572,6 @@ class _GroupPanel(object):
 
                 splitter.addWidget(panel)
 
-        # Find out how much space is available.
-        sh = splitter.sizeHint()
-        if use_widths:
-            total = sh.width()
-        else:
-            total = sh.height()
-
-        # Allocate requested space.
-        avail = total
-        remain = 0
-        for i, sz in enumerate(sizes):
-            if avail <= 0:
-                break
-
-            if sz >= 0:
-                if sz >= 1:
-                    sz = min(sz, avail)
-                else:
-                    sz *= total
-
-                sz = int(sz)
-                sizes[i] = sz
-                avail -= sz
-            else:
-                remain += 1
-
-        # Allocate the remainder to those parts that didn't request a width.
-        if remain > 0:
-            remain = int(avail / remain)
-
-            for i, sz in enumerate(sizes):
-                if sz < 0:
-                    sizes[i] = remain
-
-        splitter.setSizes(sizes)
-
     def _setup_editor(self, group, editor):
         """Setup the editor for a group.
         """
@@ -615,17 +641,11 @@ class _GroupPanel(object):
                 outer.addLayout(inner)
 
             row = 0
-
             if show_left:
                 label_alignment = QtCore.Qt.AlignRight
-                if show_labels:
-                    for i in range(1, group.columns * 2, 2):
-                        inner.setColumnStretch(i, 1)
             else:
                 label_alignment = QtCore.Qt.AlignLeft
-                if show_labels:
-                    for i in range(0, group.columns * 2, 2):
-                        inner.setColumnStretch(i, 1)
+                
         else:
             # Use the existing layout if there is one.
             if outer is None:
@@ -638,12 +658,11 @@ class _GroupPanel(object):
 
         # Process each Item in the list:
         col = -1
-
         for item in content:
+
             # Keep a track of the current logical row and column unless the
             # layout is not a grid.
             col += 1
-
             if row >= 0 and col >= columns:
                 col = 0
                 row += 1
@@ -655,20 +674,14 @@ class _GroupPanel(object):
             if name == '':
                 label = item.label
                 if label != "":
-                    # If we are building a multi-column layout with labels, 
-                    # just add space in the next column:
-                    if columns > 1 and show_labels:
-                        col += 1
 
+                    # Create the label widget.
                     if item.style == 'simple':
                         label = QtGui.QLabel(label)
                     else:
                         label = heading_text(None, text=label).control
 
-                    if row < 0:
-                        inner.addWidget(label)
-                    else:
-                        inner.addWidget(label, row, col)
+                    self._add_widget(inner, label, row, col, show_labels)
 
                     if item.emphasized:
                         self._add_emphasis(label)
@@ -700,7 +713,6 @@ class _GroupPanel(object):
                     if self.direction == QtGui.QBoxLayout.LeftToRight:
                         # Add a vertical separator:
                         line.setFrameShape(QtGui.QFrame.VLine)
-
                         if row < 0:
                             inner.addWidget(line)
                         else:
@@ -708,7 +720,6 @@ class _GroupPanel(object):
                     else:
                         # Add a horizontal separator:
                         line.setFrameShape(QtGui.QFrame.HLine)
-
                         if row < 0:
                             inner.addWidget(line)
                         else:
@@ -726,15 +737,16 @@ class _GroupPanel(object):
             # Check if it is a spacer:
             if all_digits.match( name ):
 
-                # If so, add the appropriate amount of space to the sizer:
+                # If so, add the appropriate amount of space to the layout:
                 n = int( name )
                 if self.direction == QtGui.QBoxLayout.LeftToRight:
-                    inner.Add( ( n, 1 ) )
+                    # Add a horizontal spacer:
+                    spacer = QtGui.QSpacerItem(n, 1)
                 else:
-                    spacer = ( 1, n )
-                    inner.Add( spacer )
-                    if show_labels:
-                        inner.Add( spacer )
+                    # Add a vertical spacer:
+                    spacer = QtGui.QSpacerItem(1, n)
+
+                self._add_widget(inner, spacer, row, col, show_labels)
 
                 # Continue on to the next Item in the list:
                 continue
@@ -749,7 +761,7 @@ class _GroupPanel(object):
             if item.show_label:
                 label = self._create_label(item, ui, desc)
                 self._add_widget(inner, label, row, col, show_labels,
-                        label_alignment)
+                                 label_alignment)
             else:
                 label = None
 
@@ -803,36 +815,34 @@ class _GroupPanel(object):
                 control.setFocus()
 
             # Set the correct size on the control, as specified by the user:
-            scrollable  = editor.scrollable
-            item_width  = item.width
+            stretch = 0
+            scrollable = editor.scrollable
+            item_width = item.width
             item_height = item.height
-            growable    = 0
-            # FIXME: For the moment ignore any item size until we understand
-            # what the visual effect is supposed to be (ie. we have an example).
-            item_width = item_height = -1
             if (item_width != -1) or (item_height != -1):
                 is_horizontal = (self.direction == QtGui.QBoxLayout.LeftToRight)
-                width = control.width()
-                height = control.height()
+                
+                min_size = control.minimumSizeHint()
+                width = min_size.width()
+                height = min_size.height()
 
                 if (0.0 < item_width <= 1.0) and is_horizontal:
-                    growable = int( 1000.0 * item_width )
+                    stretch = int(100 * item_width)
 
-                item_width = int( item_width )
+                item_width = int(item_width)
                 if item_width < -1:
                     item_width  = -item_width
-                    fixed_width = True
                 else:
-                    item_width = max( item_width, width )
+                    item_width = max(item_width, width)
 
                 if (0.0 < item_height <= 1.0) and (not is_horizontal):
-                    growable = int( 1000.0 * item_height )
+                    stretch = int(100 * item_height)
 
+                item_height = int(item_height)
                 if item_height < -1:
                     item_height = -item_height
-                    scrollable  = False
                 else:
-                    item_height = max( item_height, height )
+                    item_height = max(item_height, height)
 
                 control.setMinimumWidth(item_width)
                 control.setMinimumHeight(item_height)
@@ -865,17 +875,28 @@ class _GroupPanel(object):
             if item.enabled_when != '':
                 ui.add_enabled( item.enabled_when, editor )
 
-            # Add the created editor control to the sizer with the appropriate
-            # layout flags and values:
+            # Add the created editor control to the layout with the appropriate
+            # size and stretch policies:
             ui._scrollable |= scrollable
-            if item.resizable or scrollable:
-                growable = growable or 500
+            item_resizable  = ((item.resizable is True) or
+                               ((item.resizable is Undefined) and scrollable))
+            if item_resizable:
+                stretch = stretch or 50
                 self.resizable = True
-            elif item.springy:    
-                growable = growable or 500
+            elif item.springy:
+                stretch = stretch or 50
+            policy = control.sizePolicy()
+            if self.direction == QtGui.QBoxLayout.LeftToRight:
+                policy.setHorizontalStretch(stretch)
+                if item_resizable or item.springy:
+                    policy.setHorizontalPolicy(QtGui.QSizePolicy.Expanding)
+            else:
+                policy.setVerticalStretch(stretch)
+                if item_resizable or item.springy:
+                    policy.setVerticalPolicy(QtGui.QSizePolicy.Expanding)
+            control.setSizePolicy(policy)
 
-            # FIXME: Need to decide what to do about springy, border_size,
-            # padding, item.padding and growable.
+            # FIXME: Need to decide what to do about border_size and padding
             self._add_widget(inner, control, row, col, show_labels)
 
             # Save the reference to the label control (if any) in the editor:
@@ -883,7 +904,8 @@ class _GroupPanel(object):
 
         return outer
 
-    def _add_widget(self, layout, w, row, column, show_labels, label_alignment=QtCore.Qt.AlignmentFlag(0)):
+    def _add_widget(self, layout, w, row, column, show_labels, 
+                    label_alignment=QtCore.Qt.AlignmentFlag(0)):
         """Adds a widget to a layout taking into account the orientation and
            the position of any labels.
         """
@@ -894,12 +916,14 @@ class _GroupPanel(object):
             if wl is not None:
                 wl.setMargin(0)
 
+        # See if the layout is a grid.
         if row < 0:
-            # It's not a grid layout.
             if isinstance(w, QtGui.QWidget):
                 layout.addWidget(w)
-            else:
+            elif isinstance(w, QtGui.QLayout):
                 layout.addLayout(w)
+            else:
+                layout.addItem(w)
 
         else:
             if self.direction == QtGui.QBoxLayout.LeftToRight:
@@ -909,15 +933,19 @@ class _GroupPanel(object):
             if show_labels:
                 # Convert the "logical" column to a "physical" one.
                 column *= 2
-
+                    
+                # Determine whether to place widget on left or right of 
+                # "logical" column.
                 if (label_alignment != 0 and not self.group.show_left) or \
                    (label_alignment == 0 and self.group.show_left):
                     column += 1
 
             if isinstance(w, QtGui.QWidget):
                 layout.addWidget(w, row, column, label_alignment)
-            else:
+            elif isinstance(w, QtGui.QLayout):
                 layout.addLayout(w, row, column, label_alignment)
+            else:
+                layout.addItem(w, row, column, 1, 1, label_alignment)
 
     def _create_label(self, item, ui, desc, suffix = ':'):
         """Creates an item label.
@@ -957,54 +985,74 @@ class _GroupPanel(object):
 
 
 class GroupEditor(Editor):
-    """A pseudo-editor that allows a group to be managed.
+    """ A pseudo-editor that allows a group to be managed.
     """
 
     def __init__(self, **traits):
-        """Initialise the object.
+        """ Initialise the object.
         """
-
         self.set(**traits)
 
 
 class SplitterGroupEditor(GroupEditor):
-    """A pseudo-editor that allows a group with a 'split' layout to be managed.
+    """ A pseudo-editor that allows a group with a 'split' layout to be managed.
     """
 
-    #---------------------------------------------------------------------------
-    #  Trait definitions:
-    #---------------------------------------------------------------------------
-
-    # The QSplitter for the group.
-    splitter = Instance(QtGui.QSplitter)
+    # The QSplitter for the group
+    splitter = Instance(_GroupSplitter)
 
     #-- UI preference save/restore interface -----------------------------------
 
-    def restore_prefs ( self, prefs ):
-        """Restores any saved user preference information associated with the 
-           editor.
+    def restore_prefs(self, prefs):
+        """ Restores any saved user preference information associated with the 
+            editor.
         """
         if isinstance(prefs, dict):
             structure = prefs.get('structure')
         else:
             structure = prefs
 
+        self.splitter._resized = True
         self.splitter.restoreState(structure)
 
-
-    def save_prefs ( self ):
-        """Returns any user preference information associated with the editor.
+    def save_prefs(self):
+        """ Returns any user preference information associated with the editor.
         """
-        return {'structure': str(self.splitter.saveState())}
+        return { 'structure': str(self.splitter.saveState()) }
 
-    #-- End UI preference save/restore interface -------------------------------
+
+class TabbedFoldGroupEditor(GroupEditor):
+    """ A pseudo-editor that allows a group with a 'tabbed' or 'fold' layout to
+        be managed.
+    """
+    
+    # The QTabWidget or QToolBox for the group
+    container = Any
+
+    #-- UI preference save/restore interface -----------------------------------
+
+    def restore_prefs(self, prefs):
+        """ Restores any saved user preference information associated with the 
+            editor.
+        """
+        if isinstance(prefs, dict):
+            current_index = prefs.get('current_index')
+        else:
+            current_index = prefs
+        
+        self.container.setCurrentIndex(int(current_index))
+
+    def save_prefs(self):
+        """ Returns any user preference information associated with the editor.
+        """
+        return { 'current_index': str(self.container.currentIndex()) }
 
 
 #-------------------------------------------------------------------------------
 #  'HTMLHelpWindow' class:
 #-------------------------------------------------------------------------------
 
-class HTMLHelpWindow ( QtGui.QLabel ):
+class HTMLHelpWindow ( QtGui.QDialog ):
     """ Window for displaying Traits-based help text with HTML formatting.
     """
 
@@ -1015,38 +1063,31 @@ class HTMLHelpWindow ( QtGui.QLabel ):
     def __init__ ( self, parent, html, scale_dx, scale_dy ):
         """ Initializes the object.
         """
-        wx.Frame.__init__( self, parent, -1, 'Help' )
-        self.SetBackgroundColour( WindowColor )
+        QtGui.QDialog.__init__(self, parent)
+        layout = QtGui.QVBoxLayout(self)
+        layout.setMargin(0)
 
-        # Wrap the dialog around the image button panel:
-        sizer        = wx.BoxSizer( wx.VERTICAL )
-        html_control = wh.HtmlWindow( self )
-        html_control.SetBorders( 2 )
-        html_control.SetPage( html )
-        sizer.Add( html_control, 1, wx.EXPAND )
-        sizer.Add( wx.StaticLine( self, -1 ), 0, wx.EXPAND )
-        b_sizer = wx.BoxSizer( wx.HORIZONTAL )
-        button  = wx.Button( self, -1, 'OK' )
-        wx.EVT_BUTTON( self, button.GetId(), self._on_ok )
-        b_sizer.Add( button, 0 )
-        sizer.Add( b_sizer, 0, wx.ALIGN_RIGHT | wx.ALL, 5 )
-        self.SetSizer( sizer )
-        self.SetSize( wx.Size( int( scale_dx * screen_dx ), 
-                               int( scale_dy * screen_dy ) ) )
+        # Create the html control
+        html_control = QtWebKit.QWebView()
+        html_control.setSizePolicy(QtGui.QSizePolicy.Expanding, 
+                                   QtGui.QSizePolicy.Expanding)
+        html_control.setHtml(html)
+        layout.addWidget(html_control)
 
-        # Position and show the dialog:
-        position_window( self, parent = parent )
-        self.Show()
+        # Create the OK button
+        bbox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok,
+                                      QtCore.Qt.Horizontal)
+        QtCore.QObject.connect(bbox, QtCore.SIGNAL('accepted()'),
+                               self, QtCore.SLOT('accept()'))
+        layout.addWidget(bbox)
 
-    #---------------------------------------------------------------------------
-    #  Handles the window being closed:
-    #---------------------------------------------------------------------------
-
-    def _on_ok ( self, event ):
-        """ Handles the window being closed.
-        """
-        self.Destroy()
+        # Position and show the dialog
+        position_window(self, parent=parent)
+        self.show()
 
+#-------------------------------------------------------------------------------
+#  Creates a PyFace HeadingText control:
+#-------------------------------------------------------------------------------
 
 HeadingText = None
 
